from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Face, Rectangle
import rocolib.utils.numsym as np

class RectBeam(FoldedComponent):
  def define(self):
    self.addParameter("length", 100, paramType="length")
    self.addParameter("width", 20, paramType="length")
    self.addParameter("depth", 50, paramType="length")

    # XXX TODO: incorporate into minValue of parameters somehow
    self.addParameter("minlength", 0, paramType="length")
    self.addParameter("minwidth", 0, paramType="length")
    self.addParameter("mindepth", 0, paramType="length")

    self.addParameter("phase", 0, valueType="int")

    self.addParameter("angle", optional=True, overrides=("tangle", "bangle"))
    self.addParameter("tangle", 80, paramType="angle", maxValue=180)
    self.addParameter("bangle", 135, paramType="angle", maxValue=180)

    self.addParameter("root", optional=True, valueType="int")
    self.addParameter("addTabs", True, valueType="bool")

    for i in range(4):
      self.addEdgeInterface("topedge%d" % i, "r%d.e0" % i, ["width", "depth"][i % 2])
      self.addEdgeInterface("botedge%d" % i, "r%d.e2" % i, ["width", "depth"][i % 2])
      self.addFaceInterface("face%d" % i, "r%d" % i)
    self.addEdgeInterface("tabedge", "r3.e1", "length")
    self.addEdgeInterface("slotedge", "r0.e3", "length")

  def modifyParameters(self):
    self.setParameter("width", max(self.getParameter("width"), self.getParameter("minwidth")))
    self.setParameter("depth", max(self.getParameter("depth"), self.getParameter("mindepth")))
    self.setParameter("length", max(self.getParameter("length"), self.getParameter("minlength")))

  def assemble(self):
    if self.getParameter("angle") is not None:
      bangle = 90 - self.getParameter("angle")
      tangle = 90 - self.getParameter("angle")
    else:
      bangle = 90 - self.getParameter("bangle")
      tangle = 90 - self.getParameter("tangle")

    try:
      root = self.getParameter("root")
    except KeyError:
      root = None

    length = self.getParameter("length")
    width = self.getParameter("width")
    depth = self.getParameter("depth")
    phase = self.getParameter("phase")

    def dl(a):
        return np.tan(np.deg2rad(a)) * depth

    rs = []
    rs.append(Rectangle("", width, length))
    rs.append(Face("", (
      (depth, dl(tangle)),
      (depth, length - dl(bangle)),
      (0, length), 
      (0,0)
    )))
    rs.append(Rectangle("", width, length - dl(tangle) - dl(bangle)))
    rs.append(Face("", (
      (0, length), 
      (0,0),
      (depth, dl(bangle)),
      (depth, length - dl(tangle))
    )))

    for i in range(phase):
      rs.append(rs.pop(0))

    fromEdge = None
    for i in range(4):
      self.attachEdge(fromEdge, rs[i], "e3", prefix="r%d"%i, angle=90, root=((i == root) if root is not None else False))
      fromEdge = 'r%d.e1' % i
      self.setFaceInterface("face%d" % i, "r%d" % ((i-phase)%4))

    slotEdge = "r0.e3"
    tabEdge = "r3.e1"

    if self.getParameter("addTabs"):
        self.addTab(slotEdge, tabEdge, angle= 90, width=min(10, [depth, width][phase % 2]))

if __name__ == "__main__":
  RectBeam.test()
